################################################################################
#                                                                              #
# This file is distributed as part of the FIEStool data reduction package.     #
#                                                                              #
# Copyright (c) 2005 NOTSA; All Rights Reserved;                               #
# Original author: Eric Stempels; See COPYRIGHT file for details               #
#                                                                              #
################################################################################

################################################################################
#
# FIEStool
#
# checkImport.py
#
# Last update 16/01/2013 (Ricardo Cardenes)
#
################################################################################

import sys
import constants

class DependenciesError(Exception):
    pass

def numericVersion(version):
    """
    Returns the first part of a version number that comprises only numbers and
    (optionally) dots.
    """
    for candidate in version.strip().split():
        try:
            lst = [int(x) for x in candidate.split('.')]
            break
        except ValueError:
            pass
    else:
        raise ValueError('Not a valid version "%s"' % version)

    return lst

def checkImport(package, minversion):

  """
  Checks that the minimum version of required software packages is installed
  """

  try:
    thispackage = __import__(package)
  except ImportError:
    raise ImportError("Required external package, but can't import it")

  complies = True

  try:
    required = numericVersion(minversion)
    current =  numericVersion(thispackage.__version__)
    maxl = max(len(required), len(current))
    dr, dc = maxl - len(required), maxl - len(current)
    if dr > 0: required = required + ([0] * dr)
    if dc > 0: current = current + ([0] * dc)

    for x, y in zip(required, current):
        if y > x:
            break
        if x > y:
            complies = False
            break
  except ValueError:
    print "ValueError (when comparing versions for %s)" % package
    print "  - We'll try comparing substrings, which is not reliable."
    # Fallback: Compare strings, as the integer didn't work.
    #           Not reliable, but is better than nothing
    complies = not (thispackage.__version__ < minversion)


  if not complies:
    raise ImportError("Reports version '%s', but we need minimum '%s'" % (thispackage.__version__, minversion))

def checkAll():
    errors = []
    for keyword, (name, version) in constants.minversion.items():
        try:
            checkImport(keyword, version)
        except ImportError, e:
            errors.append("[%s]: %s" % (name, str(e)))
        except ValueError, e:
            errors.append("[%s]: %s" % (name, str(e)))

    if errors:
        raise DependenciesError(errors)
